@@ -27,11 +27,12 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-/* Every time the Redis Git SHA1 or Dirty status changes only this small 
+/* Every time the Redis Git SHA1 or Dirty status changes only this small
  * file is recompiled, as we access this information in all the other
  * files using this functions. */
 
 #include <string.h>
+#include <stdio.h>
 
 #include "release.h"
 #include "version.h"
@@ -50,3 +51,16 @@ uint64_t redisBuildId(void) {
 
     return crc64(0,(unsigned char*)buildid,strlen(buildid));
 }
+
+/* Return a cached value of the build string in order to avoid recomputing
+ * and converting it in hex every time: this string is shown in the INFO
+ * output that should be fast. */
+char *redisBuildIdString(void) {
+    static char buf[32];
+    static int cached = 0;
+    if (!cached) {
+        snprintf(buf,sizeof(buf),"%llx",(unsigned long long) redisBuildId());
+        cached = 1;
+    }
+    return buf;
+}
